let debug = true;;
let dprint x = if debug then MySys.print x else ();;
let dprintln x = if debug then MySys.println x else ();;


(****************************************************************************)


type l = {
    user	: int;
    sys 	: int;
    nice	: int;
    idle	: int;
  }


type load = {
    ave  : l;
    cpus : l list;
}

type t = load




let parse_line  chnl =
    let f  _ user nice sys idle = 
	{ user = user;
	  sys  = sys;
	  nice = nice;
	  idle = idle; }
    in
    Scanf.fscanf chnl "%s %d %d %d %d" f;;

let rec parse_cpus  chnl = try
    let x = parse_line chnl in
    let xs = parse_cpus chnl in
    x :: xs
with _ -> []
    
let create  () =
    let chnl = open_in "/proc/stat" in
    let load = 
	let ave  = parse_line chnl and
	    cpus = parse_cpus chnl in
	{ ave  = ave;
	  cpus = cpus; }
    in
    let () = close_in chnl in
    load;;


let pack_sub x =
    "{" ^ 
    (string_of_int x.user) ^ ";" ^
    (string_of_int x.sys) ^ ";" ^
    (string_of_int x.nice) ^ ";" ^
    (string_of_int x.idle) ^
    "}";;

let pack  x =
    "{" ^
    (pack_sub x.ave) ^ ";" ^
    (MyList.pack pack_sub x.cpus) ^ 
    "}"


let unpack_sub = function
    Tree.Leaf _ -> prerr_string "Load.unpack_sub"; failwith "Load.unpack_sub"
  | Tree.Node _ as x -> try
      { user = int_of_string (Tree.nth_elem x 0);
	sys  = int_of_string (Tree.nth_elem x 1);
	nice = int_of_string (Tree.nth_elem x 2);
	idle = int_of_string (Tree.nth_elem x 3); }
  with _ -> prerr_string "Load.unpack_sub\n"; failwith "Load.unpack_sub";;


let unpack_cpus  = function
    Tree.Leaf _ -> prerr_string "Load.unpack_cpus"; failwith "Load.unpack_cpus"
  | Tree.Node l -> try
	List.map unpack_sub l
  with _ -> prerr_string "Load.unpack_cpus\n"; failwith "Load.unpack_sub";;

let unpack = function
    Tree.Leaf _ -> prerr_string "Load.unpack"; failwith "Load.unpack"
  | Tree.Node l -> try
      { ave  = unpack_sub (List.nth l 0);
	cpus = unpack_cpus  (List.nth l 1) }
  with _ -> prerr_string "Load.unpack";  failwith "Load.unpack";;
    

let uptime  interval x1 x2 = 
    let f x1 x2 = 
	let a = (x2.user - x1.user) + (x2.sys - x1.sys) + (x2.nice - x1.nice) in
	let b = (float_of_int a) /. ((float_of_int interval) /. 1000.0) in
	if b > 100.0 then 100.0 else if b < 0.0 then 0.0 else b
    in
    let ave  = f x1.ave x2.ave and
	cpus = List.map2 f x1.cpus x2.cpus in
(*    let () = prerr_string ("ave = " ^ (string_of_float ave) ^ "\n") in
    let () = flush stderr in *)
    (ave, cpus);;

