module type OrderedType =
  sig
    type t
    val compare: t -> t -> int
  end

module type S =
  sig
    type key
    type +'a t
    val empty: 'a t
    val add: key -> 'a -> 'a t -> 'a t
    val find: key -> 'a t -> 'a
    val find_key: ('a -> 'a -> int) -> 'a -> 'a t -> key
    val remove: key -> 'a t -> 'a t
    val mem:  key -> 'a t -> bool
    val iter: (key -> 'a -> unit) -> 'a t -> unit
    val map: ('a -> 'b) -> 'a t -> 'b t
    val mapi: (key -> 'a -> 'b) -> 'a t -> 'b t
    val fold: (key -> 'a -> 'b -> 'b) -> 'a t -> 'b -> 'b
    val of_list: (key * 'a) list -> 'a t
    val to_list: 'a t -> (key * 'a) list
    val to_keys: 'a t -> key list
    val to_vals: 'a t -> 'a list
    val to_string: (key -> string) -> ('a -> string) -> 'a t -> string
  end


module Make(Ord: OrderedType) = struct
  
    module OrdMap = Map.Make(Ord)

    type key = Ord.t
    type 'a t = 'a OrdMap.t
    let empty = OrdMap.empty
    let add = OrdMap.add
    let find = OrdMap.find
    let remove = OrdMap.remove
    let mem = OrdMap.mem
    let iter = OrdMap.iter
    let map = OrdMap.map
    let mapi = OrdMap.mapi
    let fold = OrdMap.fold

    let to_list table = 
	let f key x retval = (key, x) :: retval in
	fold f table [];;
	  
    let of_list l =
	let f table (k, v) = add k v table in
	List.fold_left f empty l;;

    let to_vals table = 
	let l = to_list table in 
	List.map snd l;;

    let to_keys table = 
	let l = to_list table in 
	List.map fst l;;

    let to_string f g table =
	let l = to_list table and
	    f (k, v) = "{ key=" ^ (f k) ^ "; value=" ^ (g v) ^ " }" in
	MyList.to_string f "\n" l;;
	
    let find_key f x table = 
	let l = to_list table and
	    f (key, y) = (f x y) = 0 in 
	let (key, _) = List.find f l in
	key;;

end
