module type OrderedType =
  sig
    type t
    val compare: t -> t -> int
  end

module type S =
  sig
    type key
    type +'a t
    val empty: 'a t
    val add: key -> 'a -> 'a t -> 'a t
    val find: key -> 'a t -> 'a
    val find_key: ('a -> 'a -> int) -> 'a -> 'a t -> key
    val remove: key -> 'a t -> 'a t
    val mem:  key -> 'a t -> bool
    val iter: (key -> 'a -> unit) -> 'a t -> unit
    val map: ('a -> 'b) -> 'a t -> 'b t
    val mapi: (key -> 'a -> 'b) -> 'a t -> 'b t
    val fold: (key -> 'a -> 'b -> 'b) -> 'a t -> 'b -> 'b
    val of_list: (key * 'a) list -> 'a t
    val to_list: 'a t -> (key * 'a) list
    val to_keys: 'a t -> key list
    val to_vals: 'a t -> 'a list
    val to_string: (key -> string) -> ('a -> string) -> 'a t -> string
  end

module Make (Ord : OrderedType) : S with type key = Ord.t
