type proc = {
    pid		: Int32.t;
    comm	: string;
    state	: char;
    utime	: int;
    stime	: int;
    rss		: int;
    user	: string;
  }

type t = proc
      

let compare x y =
    Int32.compare x.pid y.pid;;
    

let parse_uid  utable pid =
    let chnl = 
	let s = "/proc/" ^ (Int32.to_string pid) ^ "/status" in
	open_in s
    in
    let x = 
	let rec f  () = (* Scan lines until a line for UID is found. *)
	    let s = input_line chnl in
	    let l = Str.split (Str.regexp "[ \t]+") s in
	    match l with
	      [] -> failwith "uid"
	    | h :: t when h = "Uid:" -> int_of_string (List.hd t)
	    | h :: t -> f ()
	in
	f ()
    in
    begin
	close_in chnl;
	Utable.find (Int32.of_int x) utable 
    end;;


let parse_stat  pid user = try
    let chnl = 
	let s = "/proc/" ^ (Int32.to_string pid) ^ "/stat" in
	open_in s
    in
    let s = input_line chnl in
    let () = close_in chnl in
    let l = Str.split (Str.regexp "[ \t+]") s in
    let comm = 
	let x = (List.nth l 1) in
	let n = (String.length x) -2 in 
	String.sub x 1 n
    in
    { pid = Int32.of_string (List.nth l 0);
      comm = comm;
      state = String.get (List.nth l 2) 0;
      utime = int_of_string (List.nth l 14);
      stime = int_of_string (List.nth l 15);
      rss   = int_of_string (List.nth l 22);
      user  = user; }
with _ -> failwith "Proc.prase_stat";;


let create  utable pid = 
    let user = parse_uid  utable pid in
    parse_stat pid user;;


let pack  x =
    "{" ^ 
    (Int32.to_string x.pid) ^ ";" ^ 
    x.comm ^ ";" ^ 
    (String.make 1 x.state) ^ ";" ^ 
    (string_of_int x.utime) ^ ";" ^ 
    (string_of_int x.stime) ^ ";" ^ 
    (string_of_int x.rss) ^ ";" ^ 
    x.user ^ 
    "}";;


let unpack =  function
    Tree.Leaf _ -> prerr_string "Proc.unpack"; failwith "Proc.unpack"
  | Tree.Node _ as x -> try
      { pid   = Int32.of_string (Tree.nth_elem x 0);
	comm  = Tree.nth_elem x 1;
	state = String.get (Tree.nth_elem x 2) 0;
	utime = int_of_string (Tree.nth_elem x 3);
	stime = int_of_string (Tree.nth_elem x 4);
	rss   = int_of_string (Tree.nth_elem x 5);
	user  = Tree.nth_elem x 6;
     }
  with _ -> prerr_string "Proc.unpack"; failwith "Proc.unpack";;
