type stat = {
    ptable	: Ptable.t;
    load	: Load.t;
  }

type t = stat


let create  utable  =
    let ptable = Ptable.create utable and
	load  = Load.create () in
    { ptable = ptable;
      load   = load; };;

let to_string x = "";;


let pack  x hostname = 
    let s1 = Ptable.pack x.ptable and
	s2 = Load.pack x.load in
    hostname ^ ";" ^
    s1 ^ ";" ^
    s2;;


let unpack = function
    Tree.Leaf _ -> prerr_string "Stat.unpack\n"; failwith "Stat.unpack"
  | Tree.Node l as x -> try
      let hostname = Tree.nth_elem x 0 and
	  stat = { ptable = Ptable.unpack (List.nth l 1);
		   load   = Load.unpack (List.nth l 2) } in
      (hostname, stat)
  with _ -> prerr_string "Stat.unpack\n"; failwith "Stat.unpack";;


let save x filename =
    let chnl = open_out filename in
    begin 
	Marshal.to_channel chnl x [];
	close_out chnl
    end;;


let load filename =
    let chnl = open_in filename in
    let x = Marshal.from_channel chnl in
    begin
	close_in chnl;
	x
    end;;


let diff_sub result (old_h, old_t) (new_h, new_t) id =
    let (old_host, (old_stat, old_time, _)) = old_h and
	(new_host, (new_stat, new_time, update_flag)) = new_h in
    let n = String.compare old_host new_host in
    if n = 0
    then 
	let result' =
	    if update_flag
	    then
		let interval = int_of_float ((new_time -. old_time) *. 1000.0) in 
		let uptime = Load.uptime interval old_stat.load new_stat.load in
		(old_host, uptime, id) :: result 
	    else
		result
	in
	(result', old_t, new_t, (id + 1))
    else if n > 0
    then (result, old_h :: old_t, new_t, (id + 1))
    else (result, old_t, new_h :: new_t, id)
	  

let diff  old_l new_l =
    let rec g  result old_l new_l id = 
	match old_l with
	  [] -> result
	| old_h :: old_t ->
	    match new_l with
	      [] -> result
	    | new_h :: new_t ->
		let (result', old_next_l, new_next_l, id') =
		    diff_sub  result (old_h, old_t) (new_h, new_t) id
		in
		g  result' old_next_l new_next_l id'
    in
    g [] old_l new_l 0;;

