type 'a tree = 
    Leaf of 'a
  | Node of ('a tree list)
type 'a t = 'a tree


let elem = function
    Leaf x -> x
  | Node _ -> failwith "Tree.elem";;


let nth_elem  tree n = 
    match tree with
      Leaf _ -> failwith "Tree.nth_elem"
    | Node l -> elem (List.nth l n);;


let rec of_string_sub  s index siblings =
    if index >= (String.length s)
    then (Node (List.rev siblings), index)
    else 
	let c = String.get s index in
(*	let () = Printf.printf "index=%d, c=%c\n" index c in *)
	match c with
	  ';' -> 
	    of_string_sub  s (index + 1) siblings
	| '(' 
	| '{' -> 
	    let (sibling, index') = of_string_sub s (index + 1) [] in
	    let siblings' = sibling :: siblings in
	    of_string_sub  s index' siblings'
	| ')' 
	| '}' -> 
	    (Node (List.rev siblings), index + 1)
	| _ -> (try
	      let i = Str.search_forward (Str.regexp "[^;{}()]+") s index in
	      let s' = Str.matched_string s in
	      let index' = i + (String.length s') in
	      let siblings' = (Leaf s') :: siblings in 
	      of_string_sub  s index' siblings'
	with Not_found -> failwith "of_string_sub");;
	

let of_string s =
    let (x, _) = of_string_sub s 0 [] in
    
    x;;


let rec print  x = 
    let rec f = function
	Leaf s -> MySys.print ("Leaf(" ^ s ^ ")")
      | Node sl -> 
	  begin
	      MySys.print "Node(";
	      List.iter (fun x -> begin f x; MySys.print "," end)  sl;
	      MySys.print ")";
	  end
    in
    f x;;
