module IntMap = MyMap.Make(Int32)
type utable = String.t IntMap.t
type t = utable


let empty = IntMap.empty;;
let add = IntMap.add;;
let find key table = try IntMap.find key table with _ -> "unknown";;
let find_key v table = try
    IntMap.find_key String.compare v table 
with _ -> failwith "Utable.find_key";;
let remove = IntMap.remove;;
let fold = IntMap.fold;;
let append x y = fold add x y;;
let of_list = IntMap.of_list;;
let to_list = IntMap.to_list;;
let to_vals = IntMap.to_vals;;
let to_keys = IntMap.to_keys;;
let to_string = IntMap.to_string Int32.to_string (function x -> x);;
 

let parse_passwd_file  chnl =
    let rec f retval =
	let s = try Some (input_line chnl) with End_of_file -> None in
	match s with
	  None -> retval
	| Some s -> 
	    let retval' = try
		let l = Str.split (Str.regexp "[:]+") s in
		let uname = List.hd l and
		    uid   = Int32.of_string (List.nth l 2) in		
		add uid uname retval
	    with _ -> retval
	    in
	    f retval'
    in
    f empty;;

let create_with_etc_passwd  () =
    let chnl = open_in "/etc/passwd" in	
    let x = parse_passwd_file chnl in
    begin
	close_in chnl;
	x
    end;;

let create_with_ypcat  () =
    let chnl = Unix.open_process_in "ypcat passwd" in 
    let x = parse_passwd_file chnl in
    begin
	ignore (Unix.close_process_in chnl);
	x
    end;;

(* Create a mapping between uid and username *)
let create  () =
    let x1 = create_with_etc_passwd () and
	x2 = create_with_ypcat () in
    append x1 x2;;
