let select_stdin  timeout =
    let fd = Unix.stdin in 
    let (fds, _, _) = UnixLabels.select ~read:[fd] ~write:[] ~except:[] ~timeout:timeout in
    List.mem fd fds;;

let wait_input () =
    select_stdin (-1.0);;

let is_readable () =
    select_stdin (0.0);;
    
let read_log  old_stable new_stable =
    let rec f  old_stable new_stable =
	if is_readable () 
	then 
	    let x = 
		let s = read_line () in
		let tree = Tree.of_string s in
		Stat.unpack tree
	    in
	    let (hostname, stat) = x in
	    let old_stable' =
		try
		    let x = Stable.find hostname new_stable in
		    Stable.add hostname x old_stable
		with _ -> old_stable
	    in
	    let new_stable' =
		let time = Unix.gettimeofday () in
		Stable.add hostname (stat, time, true) new_stable 
	    in
	    f  old_stable' new_stable'
	else (old_stable, new_stable)
    in
    begin
	(* print_string "viewer: wait\n"; flush stdout;*)
	wait_input ();
	(* print_string "viewer: wakeup\n"; flush stdout; *)
	f old_stable new_stable;
    end;;
    

let redraw  w old_stable new_stable =
    begin
	MyWidgets.draw  w old_stable new_stable; 
	Tk.update ()
    end;;


let update  w old_stable new_stable =
    let (old_stable', new_stable') = read_log  old_stable new_stable in
    let () = redraw w old_stable' new_stable' in 
    let new_stable'' = Stable.clear_update  new_stable' in
    (old_stable', new_stable'');;


let rec loop  w old_stable new_stable =
    let (old_stable, new_stable) = update w old_stable new_stable in
    loop w old_stable new_stable;;


let main  () =
    let w      = MyWidgets.create () and
	stable = Stable.empty in
    begin
	Tk.update ();
	loop w stable stable;
    end;;


let _ = main ();;
    
    
